<?php

namespace Pinpoint\Feefo\Block\Product;

use Magento\Framework\View\Element\Template;

/**
 * Class Reviews
 * @package Pinpoint\Feefo\Block\Product
 */
class Reviews extends Template
{
    /**
     * @var \Pinpoint\Feefo\Model\FeefoReview
     */
    protected $feefoReviews;

    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;

    /**
     * @var \Pinpoint\Feefo\Model\ResourceModel\FeefoReview\Collection
     */
    protected $reviews;

    /**
     * @var \Pinpoint\Feefo\Model\ResourceModel\FeefoReview\Collection
     */
    protected $reviewCollection;

    /**
     * @var \Pinpoint\Feefo\Helper\Data
     */
    protected $feefoHelper;

    /**
     * @var string
     */
    const HIDE_PRODUCT_REVIEWS_CONFIG_PATH = 'pinpoint_feefo/display_settings/product_page_reviews';

    /**
     * Reviews constructor.
     * @param \Pinpoint\Feefo\Model\FeefoReview $feefoReviews
     * @param \Pinpoint\Feefo\Helper\Data $feefoHelper
     * @param \Magento\Framework\Registry $registry
     * @param Template\Context $context
     * @param array $data
     */
    public function __construct(
        \Pinpoint\Feefo\Model\FeefoReview $feefoReviews,
        \Pinpoint\Feefo\Helper\Data $feefoHelper,
        \Magento\Framework\Registry $registry,
        Template\Context $context,
        array $data = []
    ) {
        $this->feefoReviews = $feefoReviews;
        $this->registry = $registry;
        $this->feefoHelper = $feefoHelper;
        $this->reviews = null;
        $this->reviewCollection = null;

        parent::__construct($context, $data);
    }

    /**
     * @param null $sku
     * @param bool $forceUpdate
     * @return \Pinpoint\Feefo\Model\ResourceModel\FeefoReview\Collection|mixed
     */
    public function getProductReviews($sku = null, $forceUpdate = false)
    {
        if ($this->reviews && !$forceUpdate) {
            return $this->reviews;
        }

        if (!$sku) {
            $sku = $this->registry->registry('current_product')->getSku();
        }

        $this->reviewCollection = $reviews = $this->feefoReviews->getCollection()->addFieldToFilter('product_sku', $sku)->load();
        if ($reviews->count()) {
            $this->reviews = $reviews;
            return $reviews;
        }

        return null;
    }

    /**
     * @return int
     */
    public function getReviewCount()
    {
        if (!$this->reviewCollection) {
            $this->getProductReviews();
        }

        return $this->reviewCollection->count();
    }

    /**
     * @return float|int
     */
    public function getAverageRating()
    {
        if (!$this->reviewCollection) {
            $this->getProductReviews();
        }

        $ratings = [
            '0' => 0,
            '1' => 0,
            '2' => 0,
            '3' => 0,
            '4' => 0,
            '5' => 0
        ];

        foreach ($this->reviewCollection as $review) {
            $ratings[$review->getData('rating')] += 1;
        }

        if (!array_sum(array_values($ratings))) {
            return 0;
        }

        $rating = (5*$ratings['5'] + 4*$ratings['4'] + 3*$ratings['3'] + 2*$ratings['2'] + 1*$ratings['1']) / array_sum(array_values($ratings));
        return floor($rating * 2) / 2;
    }

    /**
     * @return mixed
     */
    public function getProduct()
    {
        return $this->registry->registry('current_product');
    }

    /**
     * @return boolean
     */
    public function canShow()
    {
        return $this->feefoHelper->getConfig(self::HIDE_PRODUCT_REVIEWS_CONFIG_PATH);
    }

}